/***************************************************************************
                               cmudview.h
                             -------------------
    begin                : Sat Jun  16 22:10:48 CET 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CMUDVIEW_H
#define CMUDVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for Qt
#include <qwidget.h>
#include <qfont.h>
#include <qcolor.h>
#include <qstring.h>
#include <qsize.h>
#include <qptrlist.h>
#include <qscrollbar.h>
#include <qpixmap.h>

class CMudView : public QWidget
{
	Q_OBJECT

	class CMudViewItem {
	public:
		CMudViewItem(const QString &data, bool blink=false, const QColor &fgcolor = Qt::black, 
		             const QColor &bgcolor = Qt::white, const QFont &font = QFont::defaultFont())
			{ m_content = data; m_fgcolor = fgcolor; m_bgcolor = bgcolor; 
			  m_font = font; m_blinker = blink; m_selbegin=m_selend = -1; }
		virtual ~CMudViewItem() 
			{}
		void setContent(const QString &c)
			{ m_content  = c; }
		virtual QString getWord(const QPoint &p);
		virtual QSize size() const
			{ return QFontMetrics(m_font).size(SingleLine|ShowPrefix, m_content); }
		virtual void paint(QPainter &p);
		virtual void mouseEvent(QMouseEvent *) {}
		virtual void setSelectionStart(int begin = 0) { m_selbegin = begin; }
		virtual void setSelectionEnd(int end = 0)     { m_selend   = end;   }
		virtual void clearSelection()                 { m_selbegin = m_selend = -1; }
		virtual QString getSelection();
	protected:
		QString m_content;
		QFont m_font;
		QColor m_fgcolor, m_bgcolor;
		bool m_blinker;
		int m_selbegin, m_selend;
	};

	class CMudViewURLItem : public CMudViewItem {
	public:
		CMudViewURLItem(const QString &data, bool blink=false, const QColor &fgcolor = Qt::black, 
		                const QColor &bgcolor = Qt::white, const QFont &font = QFont::defaultFont()):
			CMudViewItem(data, blink, fgcolor, bgcolor, font)
			{ m_font.setUnderline(true); }
		void mouseEvent(QMouseEvent *);
	};

	class CMudViewImgItem : public CMudViewItem {
	public:
		CMudViewImgItem(const QString &uri): CMudViewItem(uri), m_pic(uri)
			{}
		QSize size() const 
			{ return m_pic.size(); }
		void paint(QPainter &p);
	protected:
		QPixmap m_pic;
	};

	class CMudViewLine {
	public:
		CMudViewLine()
			{ m_lineMembers.setAutoDelete(true); m_size = QSize(5, 5); }
		virtual ~CMudViewLine() 
			{}
		QSize size() const
			{ return m_size; }
		void append(const CMudViewItem *item);
		QString getWord(const QPoint &p);
		virtual void paint(QPainter &p);
		virtual void mouseEvent(QMouseEvent *e);
		virtual void setSelectionStart(int begin = 0) { m_selbegin = begin; updateSelection(); }
		virtual void setSelectionEnd(int end = 0)     { m_selend   = end;   updateSelection(); }
		virtual void clearSelection();
		virtual QString getSelection();
	protected:
		CMudViewItem *findItem(const QPoint &, int * = NULL);
		void updateSelection();
		QPtrList<CMudViewItem> m_lineMembers;
		QSize m_size;
		int m_selbegin, m_selend;
	};

public:
	/** Constructor for the main view */
	CMudView(QWidget *parent = 0, const char *name=0);
	/** Destructor for the main view */
	~CMudView();

	/** Set the defaults for Font, text color and background color.  */
	void init(const QFont &font, const QColor &textColor, const QColor &bgColor);

	/** Sets the font for all following addText operations until the next 
	    setFont() or clear() call. Status of the style setting (bold, underline, 
		italic) of the QFont object will be ignored if useStyle is false. */
	void setFont(QFont font, bool useStyle = false);
	/** Returns the currently used font, including the style settings.  */
	QFont font();
	/** Sets the underline style on if underline is true */
	void setUnderline(bool underline);
	/** Returns true if underline is active */
	bool underline();
	/** Sets the italic style on if italic is true */
	void setItalic(bool italic);
	/** Returns true if italic is active */
	bool italic();
	/** Sets the bold style on if bold is true */
	void setBold(bool bold);
	/** Returns true if bold is active */
	bool bold();
	/** Sets the text color. */
	void setTextColor(QColor color);
	/** Returns the currently used text color. */
	QColor textColor();
	/** Sets the text background color. */
	void setBgColor(QColor color);
	/** Returns the currently used text background color. */
	QColor bgColor();
	/** Returns true if the global blink state is on "draw" */
	static bool getblinkState() { return s_blinkState; }
	/** Sets the following text blinking */
	void setBlink(bool state);
	/** Returns the blink state for following text */
	bool blink();

public slots:
	/** Clears the view and resets font and color settings to the default values.
	If no default values were set, the font defaults to the QFont default, text color to red
	and background color to black. */
	void clear();

	/** Adds text to the view using the current settings for font, fontstyles (bold, underline, italic),
	text color and text background color*/
	void addText(const QString& text);

protected slots:
	virtual void resizeEvent(QResizeEvent *);
	virtual void paintEvent(QPaintEvent *);
	virtual void timerEvent(QTimerEvent *);
	virtual void wheelEvent(QWheelEvent *);
	virtual void keyPressEvent(QKeyEvent *);
	virtual void mousePressEvent(QMouseEvent *);
	virtual void mouseReleaseEvent(QMouseEvent *);
	virtual void mouseDoubleClickEvent(QMouseEvent *);
	virtual void mouseMoveEvent(QMouseEvent *);

private:
	CMudViewLine *findLine(const QPoint &, int * = NULL);
	QMouseEvent *deliverMouseEvent(QMouseEvent *);

	// the current line of text
	CMudViewLine *m_line;

	// the blink state (global to synchronize all blinkers)
	static bool s_blinkState;

	// the container for all lines
	QPtrList<CMudViewLine> m_lines;

	// current syle settings
	QFont  _font;
	QColor _textColor;
	QColor _bgColor;

	// default style settings
	QFont  _defaultFont;
	QColor _defaultTextColor;
	QColor _defaultBgColor;

	// should the following text blink?
	bool m_blink;

	// my scrollbar
	QScrollBar *m_scrollbar;
	uint m_scrollbaroffset;

	// text selection stuff
	CMudViewLine *m_startLine, *m_currLine;
	QPoint m_startPoint, m_currPoint;

};

#endif // KMUDVIEW_H

