/***************************************************************************
                               ctelnet.h
                             -------------------
    begin                : Mon May 21 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CTELNET_H
#define CTELNET_H

#include <qobject.h>
#include <qstring.h>
#include <qcstring.h>
#include <qsocket.h>

#include <kmudfilter.h>

class CMCCP;

/**
  * Class for handling a telnet connection. It takes care of telnet command
  * processing. Stuff to read: 
  * http://www.faqs.org/rfcs/rfc854.html (general telnet)
  * http://www.faqs.org/rfcs/rfc855.html (telnet options)
  * http://www.faqs.org/rfcs/rfc857.html (echo)
  * http://www.faqs.org/rfcs/rfc858.html (suppress go ahead)
  * http://www.faqs.org/rfcs/rfc1184.html (linemode)
  *
  * To make this class perfect we should keep track of the current state of
  * all telnet options. RFC says that when a request doesn't change the state
  * we must not answer (to avoid endless loops).
  * General principle of robustness: be conservative in what you do, be liberal
  * in what you accept from others.
  *
  *@author Kmud Developer Team
  */

class CTelnet : public QObject, public KmudFilter
{
	Q_OBJECT

public:
	CTelnet(QObject* parent=0, const char* name=0);
	~CTelnet();

	
	/**
	  * connect to a given host to a given port.
	  * wait max. timeout seconds for a success.
	  * returns: 0 ok, connected
	  *         -1 host not found
	  *         -2 connection refused
	  *         -3 connection timeout
	  *       -255 not connected (unknown reason)
	  */
	int connect(QString hostname, unsigned long port, unsigned int timeout=4);

	/** checks whether echo is on or off */
	bool isEchoOn();
	
	/** sets/unsets the echo, emits echoChanged() */
	void setEcho(bool b);

	/** gets the name of the host connected to */
	QString getHostName() const;

	/** gets the IP of the host connected to */
	QString getHostIP() const;

	/** gets the Port to which we are connected to */
	unsigned long getHostPort() const;

	/** true if connection is established, else false */
	bool isConnected() const;

	/** from KmudFilter. we send input to mud */
	QString filterInput(QString&);

public slots:
	/**
	  * receives data and stores it into an internal buffer. emits
	  * filledBuffer(QString) where QString is the data without the
	  * previous interpreted telnet-commands
	  */
	int receiveData();
	
	/** sends data via the telnet-connection */
	int sendData(QString buf) const;

	/** closes connection */
	void close();

signals:
	/** will be emitted when received data, QString is the data without telnet-commands */
	void filledBuffer(QString);
	
	/** emitted when echo was changed, can be done by the negotiation of the telnet-protocol */
	void changedEcho(bool);
	
	/** emitted when the connection was closed */
	void connectionClosed();
	
private:
	/** true when byte is a telnet command */
	bool isCommand(unsigned char c) const;
	/** true when byte is a telnet option */
	bool isOption(unsigned char c) const;
	/**
	  * converts a telnet code into a human readable string
	  * e.g. IAC, DO, WILL, ...
	  * when byte is neither a telnet command nor an option
	  * it is converted to the string representation of the value
	  */
	QString codeToString(unsigned char c) const;

	/**
	  * handles Telnet codes in m_buffer
	  * returns number of telnet bytes removed
	  */
	int processCommands();
	
	void timerEvent(QTimerEvent*); // reimplemented from QObject

	QSocket* m_socket;
	QByteArray m_buffer;
	bool m_echo;
	
	bool m_timerEventCalled;
	bool m_socketErrorOccured;
	QSocket::Error m_socketError;

	CMCCP*	m_mccp;

	unsigned char m_telnegState;	// current state of telnet negotiations
	QString m_telnegDebugOutput;	// human readable telnegs for debuging output
	QByteArray m_subnegs;		// bytes in subnegotiations (IAC SB ... SE)


private slots:
	void socketErrorReceived(int);
	
};

#endif

